locals {
  os_disk_name       = "os-disk"
  os_disk_size       = 128
  etcd_disk_name     = "etcd-disk"
  etcd_disk_size     = 16
  data_disk_name     = "data-disk"
  data_disk_size     = 512
  cluster_disk_name  = "cluster-disk"
  cluster_disk_size  = 256
  ceph_disk_name     = "ceph-disk"
  ceph_disk_size     = 512
  asrobots_disk_name = "asrobots-disk"
  asrobots_disk_size = 32
  os_image           = var.vm_custom_image == "" ? "rhel-cloud/rhel-8" : var.vm_custom_image
  ssh_pub_key        = file(var.ssh_pub_key_path)
  tm_instance_type   = "n2-standard-32"
}

resource "google_compute_instance" "bastion" {
  name         = "bastion-${random_string.deployment_id.result}"
  hostname     = "bastion-${random_string.deployment_id.result}.internal"
  machine_type = "e2-standard-2"

  boot_disk {
    initialize_params {
      image = "debian-cloud/debian-10"
      type  = "pd-ssd"
    }
  }

  network_interface {
    network    = google_compute_network.vnet.self_link
    subnetwork = google_compute_subnetwork.vnet-subnet.self_link
    access_config {
      network_tier = "STANDARD"
    }
  }

  metadata = {
    "ssh-keys" = "${var.ssh_user}:${local.ssh_pub_key}"
  }

  depends_on = [
    google_compute_region_instance_group_manager.mig-servers,
    google_compute_region_instance_group_manager.mig-agents,
    google_compute_region_instance_group_manager.mig-asrobots-agents,
    google_compute_region_instance_group_manager.mig-gpus[0]
  ]

  service_account {
    email  = data.google_compute_default_service_account.default.email
    scopes = ["cloud-platform"]
  }
}

resource "null_resource" "prepare_installation" {
  provisioner "file" {
    connection {
      host        = google_compute_instance.bastion.network_interface[0].access_config[0].nat_ip
      type        = "ssh"
      user        = var.ssh_user
      private_key = file(var.ssh_private_key_path)
    }
    destination = "/home/${var.ssh_user}/.ssh/private_key"
    source      = var.ssh_private_key_path
  }

  provisioner "remote-exec" {
    connection {
      host        = google_compute_instance.bastion.network_interface[0].access_config[0].nat_ip
      type        = "ssh"
      user        = var.ssh_user
      private_key = file(var.ssh_private_key_path)
    }
    inline = [
      "sudo chmod 400 .ssh/private_key"
    ]
  }
  depends_on  = [ google_compute_instance.bastion ]
}

resource "null_resource" "wait_for_installation" {
  connection {
    host        = google_compute_instance.bastion.network_interface[0].access_config[0].nat_ip
    type        = "ssh"
    user        = var.ssh_user
    private_key = file(var.ssh_private_key_path)
  }

  provisioner "file" {
    destination = "/home/${var.ssh_user}/wait_for_installation.sh"
    source      = "scripts/wait_for_installation.sh"
  }

  provisioner "remote-exec" {
    inline = [
      "sudo chmod +x /home/${var.ssh_user}/wait_for_installation.sh",
      "/home/${var.ssh_user}/wait_for_installation.sh ${var.region} ${google_compute_region_instance_group_manager.mig-servers.name} ${var.ssh_user} ${var.perform_installation} || exit 1",
      "rm -f /home/${var.ssh_user}/wait_for_installation.sh"
    ]
    on_failure = fail
  }
  depends_on = [ google_compute_instance.bastion, null_resource.prepare_installation ]
}

resource "google_compute_region_instance_group_manager" "mig-servers" {
  name               = "servers-manager-${random_string.deployment_id.result}"
  base_instance_name = "server-${random_string.deployment_id.result}"
  target_size        = var.server_count
  wait_for_instances = true
  region             = var.region

  version {
    name              = "servers"
    instance_template = google_compute_instance_template.server.id
  }

  named_port {
    name = "https"
    port = 443
  }

  named_port {
    name = "kube"
    port = 6443
  }

  named_port {
    name = "rke2"
    port = 9345
  }
}

resource "google_compute_region_instance_group_manager" "mig-agents" {
  name               = "agents-manager-${random_string.deployment_id.result}"
  base_instance_name = "agent-${random_string.deployment_id.result}"
  target_size        = var.agent_count
  wait_for_instances = true
  region             = var.region

  version {
    name              = "agents"
    instance_template = google_compute_instance_template.agent.id
  }

  named_port {
    name = "https"
    port = 443
  }
  depends_on = [
    google_compute_region_instance_group_manager.mig-servers
  ]
}

resource "google_compute_region_instance_group_manager" "mig-asrobots-agents" {
  name               = "asrobots-agents-manager-${random_string.deployment_id.result}"
  base_instance_name = "asrobots-${random_string.deployment_id.result}"
  target_size        = var.asrobots_agent_count
  wait_for_instances = true
  region             = var.region

  version {
    name              = "asrobots"
    instance_template = google_compute_instance_template.asrobots-agent.id
  }

  named_port {
    name = "https"
    port = 443
  }
  depends_on = [
    google_compute_region_instance_group_manager.mig-servers
  ]
}

resource "google_compute_region_instance_group_manager" "mig-gpus" {
  count              = (var.gpu_agent_count > 0) ? 1 : 0
  name               = "gpu-manager-${random_string.deployment_id.result}"
  base_instance_name = "gpu-agent-${random_string.deployment_id.result}"
  target_size        = var.gpu_agent_count
  wait_for_instances = true
  region             = var.region

  version {
    name              = "gpus"
    instance_template = google_compute_instance_template.agent-gpu[0].id
  }

  named_port {
    name = "https"
    port = 443
  }
  depends_on = [
    google_compute_region_instance_group_manager.mig-servers
  ]
}

resource "google_compute_instance_template" "server" {
  name         = "server-template-${random_string.deployment_id.result}"
  machine_type = var.server_instance_type
  region       = var.region
  disk {
    device_name  = local.os_disk_name
    boot         = true
    auto_delete  = true
    source_image = local.os_image
    disk_size_gb = local.os_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  disk {
    device_name  = local.cluster_disk_name
    auto_delete  = true
    disk_size_gb = local.cluster_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  disk {
    device_name  = local.data_disk_name
    auto_delete  = true
    disk_size_gb = local.data_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  disk {
    device_name  = local.etcd_disk_name
    auto_delete  = true
    disk_size_gb = local.etcd_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  disk {
    device_name  = local.ceph_disk_name
    auto_delete  = true
    disk_size_gb = local.ceph_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  dynamic "disk" {
    for_each = (var.server_count == 1 && var.install_as_robots) ? [1] : []
    content {
      device_name  = local.asrobots_disk_name
      auto_delete  = true
      disk_size_gb = local.asrobots_disk_size
      type         = "PERSISTENT"
      disk_type    = "pd-ssd"
    }
  }

  network_interface {
    network    = google_compute_network.vnet.self_link
    subnetwork = google_compute_subnetwork.vnet-subnet.self_link

  }


  metadata = {
    "ssh-keys"                = "${var.ssh_user}:${local.ssh_pub_key}"
    "enable-guest-attributes" = "TRUE"
  }

  metadata_startup_script = (var.perform_installation ? data.template_file.install-server.rendered : data.template_file.install-prereqs.rendered)
  lifecycle {
    ignore_changes = [
      metadata_startup_script
    ]
  }

  service_account {
    email = data.google_compute_default_service_account.default.email
    scopes = [
      "cloud-platform"
    ]
  }
}

resource "google_compute_instance_template" "agent" {
  name         = "agent-template-${random_string.deployment_id.result}"
  machine_type = var.agent_instance_type
  region       = var.region
  disk {
    device_name  = local.os_disk_name
    boot         = true
    auto_delete  = true
    source_image = local.os_image
    disk_size_gb = local.os_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  disk {
    device_name  = local.cluster_disk_name
    auto_delete  = true
    disk_size_gb = local.cluster_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  network_interface {
    network    = google_compute_network.vnet.self_link
    subnetwork = google_compute_subnetwork.vnet-subnet.self_link
  }


  metadata = {
    "ssh-keys"                = "${var.ssh_user}:${local.ssh_pub_key}"
    "enable-guest-attributes" = "TRUE"
  }

  metadata_startup_script = (var.perform_installation ? data.template_file.install-agent.rendered : data.template_file.install-prereqs.rendered)
  lifecycle {
    ignore_changes = [
      metadata_startup_script
    ]
  }

  service_account {
    email = data.google_compute_default_service_account.default.email
    scopes = [
      "cloud-platform"
    ]
  }
}
resource "google_compute_instance_template" "asrobots-agent" {
  name         = "asrobots-template-${random_string.deployment_id.result}"
  machine_type = var.asrobots_agent_instance_type
  region       = var.region
  disk {
    device_name  = local.os_disk_name
    boot         = true
    auto_delete  = true
    source_image = local.os_image
    disk_size_gb = local.os_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  disk {
    device_name  = local.cluster_disk_name
    auto_delete  = true
    disk_size_gb = local.cluster_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  disk {
    device_name  = local.asrobots_disk_name
    auto_delete  = true
    disk_size_gb = local.asrobots_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }


  network_interface {
    network    = google_compute_network.vnet.self_link
    subnetwork = google_compute_subnetwork.vnet-subnet.self_link
  }


  metadata = {
    "ssh-keys"                = "${var.ssh_user}:${local.ssh_pub_key}"
    "enable-guest-attributes" = "TRUE"
  }

  metadata_startup_script = (var.perform_installation ? data.template_file.install-agent-asrobots.rendered : data.template_file.install-prereqs.rendered)
  lifecycle {
    ignore_changes = [
      metadata_startup_script
    ]
  }

  service_account {
    email = data.google_compute_default_service_account.default.email
    scopes = [
      "cloud-platform"
    ]
  }
}

resource "google_compute_instance_template" "agent-gpu" {
  count        = (var.gpu_agent_count > 0) ? 1 : 0
  name         = "agent-gpu-template-${random_string.deployment_id.result}"
  machine_type = var.gpu_instance_type
  region       = var.region
  guest_accelerator {
    count = 1
    type  = var.gpu_type
  }
  disk {
    device_name  = local.os_disk_name
    boot         = true
    auto_delete  = true
    source_image = local.os_image
    disk_size_gb = local.os_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  disk {
    device_name  = local.cluster_disk_name
    auto_delete  = true
    disk_size_gb = local.cluster_disk_size
    type         = "PERSISTENT"
    disk_type    = "pd-ssd"
  }

  network_interface {
    network    = google_compute_network.vnet.self_link
    subnetwork = google_compute_subnetwork.vnet-subnet.self_link
  }

  metadata = {
    "ssh-keys"                = "${var.ssh_user}:${local.ssh_pub_key}"
    "enable-guest-attributes" = "TRUE"
  }

  metadata_startup_script = (var.perform_installation ? data.template_file.install-agent-gpu.rendered : data.template_file.install-prereqs-gpu.rendered)

  lifecycle {
    ignore_changes = [
      metadata_startup_script
    ]
  }
  scheduling {
    on_host_maintenance = "TERMINATE"
  }
  service_account {
    email = data.google_compute_default_service_account.default.email
    scopes = [
      "cloud-platform"
    ]
  }
}
resource "google_compute_instance" "tm-node" {
  name         = "tm-node-${random_string.deployment_id.result}"
  machine_type = local.tm_instance_type
  hostname     = "tm-node-${random_string.deployment_id.result}.internal"
  count        = var.install_task_mining ? 1 : 0

  boot_disk {
    device_name = local.os_disk_name
    auto_delete = true
    initialize_params {
      image = local.os_image
      size  = local.os_disk_size
      type  = "pd-ssd"
    }
  }

  attached_disk {
    device_name = local.cluster_disk_name
    source      = google_compute_disk.tm-cluster-disk[0].self_link
  }

  network_interface {
    network    = google_compute_network.vnet.self_link
    subnetwork = google_compute_subnetwork.vnet-subnet.self_link
  }

  metadata = {
    "ssh-keys"                = "${var.ssh_user}:${local.ssh_pub_key}"
    "enable-guest-attributes" = "TRUE"
  }

  metadata_startup_script = (var.perform_installation ? data.template_file.install-agent-tm.rendered : data.template_file.install-prereqs.rendered)
  lifecycle {
    ignore_changes = [
      metadata_startup_script
    ]
  }

  depends_on = [
    google_compute_region_instance_group_manager.mig-servers
  ]

  service_account {
    email = data.google_compute_default_service_account.default.email
    scopes = [
      "cloud-platform"
    ]
  }
}

resource "google_compute_disk" "tm-cluster-disk" {
  count = var.install_task_mining ? 1 : 0
  name  = "tm-${random_string.deployment_id.result}-cluster-disk"
  size  = local.cluster_disk_size
  type  = "pd-ssd"
}
