locals {
  input_common_parameters = {
    fqdn                  = var.lb_fqdn
    fixed_address         = google_compute_address.lb-proxies-ip.address
    multinode             = (var.server_count >= 3)
    ha                    = (var.server_count >= 3)
    profile               = ((var.server_count >= 3) ? "ha" : "default")
    gpu_support           = (var.gpu_agent_count > 0) ? true : false
    rke_token             = uuid()
    db_ip                 = google_sql_database_instance.sql_instance.ip_address[0].ip_address
    orchestrator          = var.install_orchestrator
    aicenter              = var.install_aicenter
    documentunderstanding = var.install_documentunderstanding
    insights              = var.install_insights
    dataservice           = var.install_data_service
    test_manager          = var.install_test_manager
    automation_ops        = var.install_automation_ops
    automation_hub        = var.install_automation_hub
    apps                  = var.install_apps
    action_center         = var.install_action_center
    task_mining           = var.install_task_mining
    process_mining        = var.install_process_mining
    asrobots              = var.install_as_robots
  }

  prereqs_common_parameters = {
    installer_version                  = var.installer_version
    installer_url                      = var.installer_url != "" ? var.installer_url : "https://download.uipath.com/automation-suite/${var.installer_version}/installer-${var.installer_version}.zip"
    region                             = var.region
    instance_group_name                = "servers-manager-${random_string.deployment_id.result}"
    as_default_credentials_secret_name = google_secret_manager_secret.as-default-credentials-secret.secret_id
    db_credentials_secret_name         = google_secret_manager_secret.db-credentials-secret.secret_id
    argo_credentials_secret_name       = google_secret_manager_secret.argo-credentials-secret.secret_id
    argo_user_credentials_secret_name  = google_secret_manager_secret.argo-user-credentials-secret.secret_id
    install_aicenter                   = var.install_aicenter
    install_apps                       = var.install_apps
    install_documentunderstanding      = var.install_documentunderstanding
    install_task_mining                = var.install_task_mining
    is_production_pm                    = local.is_production_pm ? "true" : "false"
    pm_sql_instance_ip                 = local.is_production_pm ? google_sql_database_instance.pm_sql_instance[0].ip_address[0].ip_address : ""
    as_robots_disk_size                = local.asrobots_disk_size
  }
}
data "template_file" "install-server" {
  template = file("scripts/install_server.sh")

  vars = {
    install_prereqs = data.template_file.install-prereqs.rendered
    number_of_nodes = (var.server_count + var.agent_count + var.asrobots_agent_count + var.gpu_agent_count + (var.install_task_mining ? 1 : 0))
  }
}

data "template_file" "install-agent" {
  template = file("scripts/install_agent.sh")

  vars = {
    install_prereqs = data.template_file.install-prereqs.rendered
    node_type       = "agent"
  }
}

data "template_file" "install-agent-asrobots" {
  template = file("scripts/install_agent.sh")

  vars = {
    install_prereqs = data.template_file.install-prereqs.rendered
    node_type       = "asrobots"
  }
}

data "template_file" "install-agent-tm" {
  template = file("scripts/install_agent.sh")

  vars = {
    install_prereqs = data.template_file.install-prereqs.rendered
    node_type       = "task-mining"
  }
}

data "template_file" "install-prereqs" {
  template = file("scripts/install_prereqs")
  vars     = merge(local.prereqs_common_parameters, { gpu_prereqs = "echo 'No GPU prereqs'", cluster_config = data.template_file.inputs-non-gpu.rendered })
}

data "template_file" "inputs-non-gpu" {
  template = file("scripts/inputs_schema")
  vars     = local.input_common_parameters
}

data "template_file" "install-agent-gpu" {
  template = file("scripts/install_agent.sh")

  vars = {
    install_prereqs = data.template_file.install-prereqs-gpu.rendered
    node_type       = "gpu"
  }
}

data "template_file" "install-prereqs-gpu" {
  template = file("scripts/install_prereqs")
  vars     = merge(local.prereqs_common_parameters, { gpu_prereqs = file("scripts/install_prereqs_gpu"), cluster_config = data.template_file.inputs-gpu.rendered })
}

data "template_file" "inputs-gpu" {
  template = file("scripts/inputs_schema")
  vars     = local.input_common_parameters
}
data "template_file" "init-proxy" {
  template = file("scripts/init_proxy.sh")
  vars = {
    "servers_lb_ip" = google_compute_address.lb-registration-ip.address
    "proxies_lb_ip" = google_compute_address.lb-proxies-ip.address
  }
}
