${install_prereqs}

echo "Starting installation process"
NODE_TYPE="FIRST_SERVER"
LOCATION_IN_ARRAY=0

export PATH=$PATH:/var/lib/rancher/rke2/bin:/opt/UiPathAutomationSuite/Installer/bin:/usr/local/bin
export KUBECONFIG="/etc/rancher/rke2/rke2.yaml"

function fail_and_exit() {
  curl -s -X PUT --data "FAILED" http://metadata.google.internal/computeMetadata/v1/instance/guest-attributes/installation/app_installer -H "Metadata-Flavor: Google"
  exit 1
}

function get_first_server() {
  local instance_group
  local instance_id

  instance_id=$(curl -s "http://metadata.google.internal/computeMetadata/v1/instance/id" -H "Metadata-Flavor: Google")
  instance_group=$(curl -s "http://metadata.google.internal/computeMetadata/v1/instance/attributes/created-by" -H "Metadata-Flavor: Google")
  echo "Local instance id: $instance_id in group $instance_group"
  
  servers_list=$(gcloud compute instances list --filter="metadata.items.key['created-by']['value']=$instance_group" --sort-by=INTERNAL_IP --format="table[no-heading](ID, INTERNAL_IP)"  | awk '{print $1}')
  arr_servers_list=($servers_list)
  first_server=$${arr_servers_list[0]}
  LOCATION_IN_ARRAY=-1
  for crt_id in "$${arr_servers_list[@]}"; do
    echo "$instance_id vs $crt_id"
    [[ $crt_id == "$instance_id" ]] && break
    (( ++LOCATION_IN_ARRAY ))
  done

  echo "Current instance: $instance_id | First server instance: $first_server"

  if [[ "$instance_id" == "$first_server" ]]; then
    echo "Current node is the first server"
    NODE_TYPE="FIRST_SERVER"
  else
    echo "Current node is a secondary server  | Position in queue for installation $LOCATION_IN_ARRAY"
    NODE_TYPE="SECONDARY_SERVER"
  fi
}

function check_registration() {
  local registration_url=$(cat /opt/UiPathAutomationSuite/Installer/cluster_config.json | jq -r '.fixed_rke_address')
  local registration_status

  registration_status=""
  local try=0
  local maxtry=5

  while [[ "$registration_status" != "200" ]] && ((try != maxtry)); do
    try=$((try + 1))
    registration_status=$(curl -w '%%{response_code}' -sk -o /dev/null https://"$registration_url":9345/ping) || true
    echo "Trying to reach $registration_url ==== $try/$maxtry" && sleep 2
  done

  if [[ "$registration_status" -ne "200" ]]; then
    echo "No RKE2 server listening at the registration address"
    get_first_server
  else
    echo "There is already an RKE2 server listening at the registration address"
    NODE_TYPE="SECONDARY_SERVER"
  fi
}

function install_uipath() { 
  local registration_url
  local no_of_nodes_actual
  local no_of_nodes_required

  registration_url="$(jq -r ".fixed_rke_address" <"/opt/UiPathAutomationSuite/Installer/cluster_config.json")"

  check_registration "$registration_url"

  if [[ "$NODE_TYPE" == "FIRST_SERVER" ]]; then
    echo "Installing as primary server"

    try=0
    maxtry=5
    status="notready"
    while [[ "$status" != "ready" ]] && ((try != maxtry)); do
      try=$((try + 1))
      echo "Trying to install infra ==== $try/$maxtry"
      /opt/UiPathAutomationSuite/Installer/install-uipath.sh -i /opt/UiPathAutomationSuite/Installer/cluster_config.json -o /opt/UiPathAutomationSuite/Installer/output.json -k --accept-license-agreement && status="ready"
      sleep 30
    done
    set -eu
    [[ "$status" == "ready" ]] || (echo "Failed to install infra" && fail_and_exit)
    set +eu
    echo "Successfuly installed infra, waiting for the other nodes"

    no_of_nodes_required="$1"
    nodereadypath='{range .items[*]}{@.metadata.name}:{range @.status.conditions[*]}{@.type}={@.status}{end}{"\n"}{end}'

    local try=0
    local maxtry=60
    no_of_nodes_actual=$(kubectl get nodes -o jsonpath="$nodereadypath" | grep -c -E "Ready=True")

    while (( no_of_nodes_required != no_of_nodes_actual )) && ((try != maxtry)); do
      try=$((try + 1))
      echo "Waiting for $no_of_nodes_required to be ready, current nodes $no_of_nodes_actual ==== $try/$maxtry" && sleep 30
      no_of_nodes_actual=$(kubectl get nodes -o jsonpath="$nodereadypath" | grep -c -E "Ready=True")
    done

    set -eu
    (( no_of_nodes_required == no_of_nodes_actual )) || (echo "Required number of nodes failed to start" && fail_and_exit)
    set +eu

    try=0
    maxtry=4
    status="notready"
    while [[ "$status" != "ready" ]] && ((try != maxtry)); do
      try=$((try + 1))
      echo "Trying to install the fabric ==== $try/$maxtry"
      /opt/UiPathAutomationSuite/Installer/install-uipath.sh -i /opt/UiPathAutomationSuite/Installer/cluster_config.json -o /opt/UiPathAutomationSuite/Installer/output.json -f --accept-license-agreement && status="ready"
      sleep 30
    done
    set -eu
    [[ "$status" == "ready" ]] || (echo "Failed to install fabric" && fail_and_exit)
    set +eu

    try=0
    maxtry=4
    status="notready"
    while [[ "$status" != "ready" ]] && ((try != maxtry)); do
      try=$((try + 1))
      echo "Trying to install the services ==== $try/$maxtry"
      /opt/UiPathAutomationSuite/Installer/install-uipath.sh -i /opt/UiPathAutomationSuite/Installer/cluster_config.json -o /opt/UiPathAutomationSuite/Installer/output.json -s --accept-license-agreement && status="ready"
      sleep 30
    done
    set -eu
    [[ "$status" == "ready" ]] || (echo "Failed to install services" && fail_and_exit)
    set +eu
    if [[ $(cat /opt/UiPathAutomationSuite/Installer/cluster_config.json | jq -r '.gpu_support' ) == 'true' ]]; then
      echo "Enabling gpu for cluster..."
      /opt/UiPathAutomationSuite/Installer/configureUiPathAS.sh gpu enable
    fi
  else
    echo "Installing as secondary server"

    registration_status=""
    local try=0
    local maxtry=60

    registration_status=$(curl -w '%%{response_code}' -sk -o /dev/null https://"$registration_url":9345/ping)  || true
    while [[ "$registration_status" != "200" ]] && ((try != maxtry)); do
      try=$((try + 1))
      registration_status=$(curl -w '%%{response_code}' -sk -o /dev/null https://"$registration_url":9345/ping)  || true
      echo "Trying to reach $registration_url ==== $try/$maxtry" && sleep 30
    done
    set -eu
    [[ "$registration_status" == "200" ]] || (echo "Primary server failed to start" && fail_and_exit)
    set +eu
    echo "Primary server is up. Waiting for components to install on primary server. Position in array of current server is: $LOCATION_IN_ARRAY"
    echo "Primary server installed all components"
    # introducing 2 minutes between servers, to avoid the etcd too many learners race condition
    sleep $(( LOCATION_IN_ARRAY*120 ))

    try=0
    maxtry=5
    status="notready"
    while [[ "$status" != "ready" ]] && ((try != maxtry)); do
      try=$((try + 1))
      echo "Trying to install infra ==== $try/$maxtry"
      /opt/UiPathAutomationSuite/Installer/install-uipath.sh -i /opt/UiPathAutomationSuite/Installer/cluster_config.json -o /opt/UiPathAutomationSuite/Installer/output.json -k -j server --accept-license-agreement && status="ready"
      sleep 30
    done
    if [[ "$status" == "notready" ]]; then
      echo "restarting the RKE2 server"
      systemctl restart rke2-server.service

      try=0
      maxtry=90

      while [[ $(kubectl get --raw='/readyz') != "ok" ]] && (( try != maxtry )); do
        try=$((try+1))
        echo "waiting for kubernetes api server to be ready...$try/$maxtry" &&  sleep 10;
      done

      try=0
      while [[ $(kubectl get node "$(hostname)" -o 'jsonpath={..status.conditions[?(@.type=="Ready")].status}') =~ "False" ]] && (( try != maxtry )); do
        try=$((try+1))
        echo "waiting for server to be ready...$try/$maxtry" &&  sleep 10;
      done
      set -eu
      kubectl cluster-info &>/dev/null || (echo "Could not reach the cluster API" && fail_and_exit)
      set +eu
    fi
  fi
  curl -s -X PUT --data "SUCCESS" http://metadata.google.internal/computeMetadata/v1/instance/guest-attributes/installation/app_installer -H "Metadata-Flavor: Google"
}

install_status=$(curl -s http://metadata.google.internal/computeMetadata/v1/instance/guest-attributes/installation/app_installer -H "Metadata-Flavor: Google")
if [ "$install_status" != "SUCCESS" ]; then
    install_uipath ${number_of_nodes} > /opt/UiPathAutomationSuite/install.logs 2>&1
else
    echo "Installer already run"
fi
